// -*-c++-*-

/*
 *Copyright:

 Copyright (C) Hidehisa AKIYAMA

 This code is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 This code is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this code; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

 *EndCopyright:
 */

/////////////////////////////////////////////////////////////////////

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "bhv_basic_tackle.h"
#include "bhv_dangerous.h"

#include <rcsc/action/neck_turn_to_ball_or_scan.h>

#include <rcsc/player/player_agent.h>
#include <rcsc/player/debug_client.h>

#include <rcsc/common/logger.h>
#include <rcsc/common/server_param.h>

/*-------------------------------------------------------------------*/
/*!
  execute action
*/
bool
Bhv_Dangerous::execute( rcsc::PlayerAgent * agent )
{
    const rcsc::WorldModel & wm = agent->world();

    if ( agent->config().version() >= 12.0 )
      {
	/*
	  rcsc::dlog.addText( rcsc::Logger::TEAM,
	  "%s:%d: Bhv_Da. to body dir"
	  ,__FILE__, __LINE__ );
	  agent->debugClient().addMessage( "Tackle+" );
	  agent->doTackle( 0.0 ); // forward tackle
	*/
	
	PlayerObject * opp = getMarkTarget( wm, M_home_pos ) ;
	rcsc::AngleDeg block_angle = ( wm.ball().pos() - opp->pos() ).th() ;
	rcsc::Vector2D block_point 
	  = opp->pos() + rcsc::Vector2D::polar2vector( 0.2, block_angle ) ;
	
	block_point.x -= 0.1 ;
	
	if ( block_point.x < wm.self().pos().x - 1.5 ) {
	  block_point.y = wm.self().pos().y ;
	}
	
	double dist = ( block_point - wm.self().pos() ).r() ;
	rcsc::Body_GoToPoint( block_point,
			      1.0,
			      rcsc::ServerParam::i().maxPower(),
			      5,
			      false,
			      true,
			      40.0,
			      ).execute( agent ) ;	
	agent->setNeckAction( new rcsc::Neck_TurnToBallOrScan() );        
      }
    else // v11 or older
      {
	double tackle_power = rcsc::ServerParam::i().maxTacklePower();
	if ( wm.self().body().abs() < M_body_thr )
	  {
	    rcsc::dlog.addText( rcsc::Logger::TEAM,
				"%s:%d: Bhv_BasicTackle. to body dir"
				,__FILE__, __LINE__ );
	    
	    agent->debugClient().addMessage( "Tackle+" );
	    agent->doTackle( tackle_power );
	    agent->setNeckAction( new rcsc::Neck_TurnToBallOrScan() );
	    return true;
	  }
	
	tackle_power = - rcsc::ServerParam::i().maxBackTacklePower();
	if ( tackle_power < 0.0
	     && wm.self().body().abs() > 180.0 - M_body_thr )
	  {
	    rcsc::dlog.addText( rcsc::Logger::TEAM,
				"%s:%d: Bhv_BasicTackle. to body reverse dir"
				,__FILE__, __LINE__ );
	    
	    agent->debugClient().addMessage( "Tackle-" );
	    agent->doTackle( tackle_power );
	    agent->setNeckAction( new rcsc::Neck_TurnToBallOrScan() );
	    return true;
	  }
      }
}

return false;
}

const rcss::PlayerObject *
getMarkTarget( const rcss::WorldModel & wm,
	       const rcss::Vector2D & home_pos ) {
  double dist_opp_to_home = 100.0 ;
  const PlayerObject * opp 
    = wm.getOpponentNearestTo( M_home_pos, 1, &dist_opp_to_home ) ;
  
  if ( ! opp
       || ( wm.existKickableTeammate()
	    &&  opp->distFromBall() > 2.5 ) ) {
    return NULL ;
  }
  
  if ( dist_opp_to_home > 7.0 && home_pos.x  < opp->pos().x ) {
    return NULL ;
  }
  
  const PlayerPtrCont::const_iterator
    end = wm.getTeammatesFromSelf().end() ;
  for ( PlayerPtrCont::const_iterator ) 
    it = wm.getTeammatesFromSelf().begin() ;
  it != end ;
  ++it ) {
  if ( (*it ) -> pos().dist( op->pos() ) < dist_opp_to_home ) {
    return NULL ;
  }
}
return opp ;
}

  
  
       
    
